FilterTradeSkillDB = {}


local DEFAULTS = {
    ShownSkills = 8,
    locked = true,
    movable = false,
    
    optimal = true,
    medium = true,
    easy = true,
    trivial = true,
    
    name = true,
    desc = true,
    mats = true,
    
    noregex = true,
    
    yOffset = 0,
}


function FilterTradeSkill:OnLoad()
    
    AceHookLib:Embed(self, "1.3")
    
    self.topleft = FilterTradeSkill_TopLeft
    self.topright = FilterTradeSkill_TopRight
    self.tooltip = FilterTradeSkillTT
    
    self.lockbutton = FilterTradeSkillLockButton
    self.lockbutton.normal = FilterTradeSkillLockNormal
    self.lockbutton.pushed = FilterTradeSkillLockPushed
    
    FilterTradeSkill_TopLeft = nil
    FilterTradeSkill_TopRight = nil
    
    self.NOOP = function() end
    self.defaults = DEFAULTS
    self.filter = ""
    self.showState = setmetatable({},{__index = function() return true end})
    self.anchor = {}
    
    self:SetScript("OnEvent",self.OnEvent)
    self:SetScript("OnShow",self.OnShow)
    self:SetScript("OnHide",self.OnHide)
    
    self:RegisterEvent("ADDON_LOADED")
    self:RegisterEvent("TRADE_SKILL_SHOW")
    self:doHook()
    
end
function FilterTradeSkill:SetDefaults()
    for k,v in pairs(self.defaults) do
        if(self.db[k] == nil) then
            self.db[k] = v
        end
    end
end

function FilterTradeSkill:doHook()
    
    self:Hook("TradeSkillFrame_Update","Update")
    --hooksecurefunc("TradeSkillFrame_Update",function()self:Update()end)
    self:HookScript(TradeSkillFrame,"OnMouseDown")
    self:HookScript(TradeSkillFrame,"OnMouseUp")
    
    --self:HookScript(TradeSkillFrame,"OnHide")
    --self:HookScript(TradeSkillFrame,"OnShow") 
    
    self:Hook("ExpandTradeSkillSubClass")
    self:Hook("CollapseTradeSkillSubClass")

end

function FilterTradeSkill:ApplySettings()
    
    TradeSkillExpandButtonFrame:SetPoint("TOPLEFT",15,-101)
    TradeSkillInvSlotDropDown:SetPoint("TOPRIGHT",-25,-96)
    TradeSkillListScrollFrame:SetPoint("TOPRIGHT",-67,-126)
    TradeSkillSkill1:SetPoint("TOPLEFT",26,-126)
    
    TradeSkillFrame:DisableDrawLayer("BORDER")
    TradeSkillFrameTitleText[0] = FilterTradeSkillTitleText[0]
        
    self:SetFrameLevel(TradeSkillFrame:GetFrameLevel()-1)
    
    self:Resize(self.db.ShownSkills)
    self:UpdateLockButton()
    
    self:SetUIPanel(TradeSkillFrame,nil)
    table.insert(UISpecialFrames,"TradeSkillFrame")
    table.insert(UISpecialFrames,"FilterTradeSkillConfigFrame")
    self:UpdateMovable()
    
    if(FAC_Config) then FAC_Config.NoOverride = true end -- This is to keep AutoCraft from messing up the display if it's present
end

function FilterTradeSkill:UpdateMovable()
    if(not self.db) then return end
    if(not self.db.movable) then
        self:SetUIPanel(FilterTradeSkillAnchorFrame,{area = "left", pushable = 1})
        FilterTradeSkill:AttachToAnchorFrame()
        self.offsetSlider:Enable()
    else
        self:SetUIPanel(FilterTradeSkillAnchorFrame,nil)
        self:ResetFramePosition()
        self.offsetSlider:Disable()
    end
    
end
function FilterTradeSkill:AttachToAnchorFrame()
    TradeSkillFrame:SetPoint("TOPLEFT",FilterTradeSkillAnchorFrame,0,0 + self.db.yOffset)
end
function FilterTradeSkill:SetUIPanel(f,info)
    local n = f:GetName()
    local visible = f:IsVisible()
    if(visible) then
        self:HookScript(f,"OnShow","NOOP")
        self:HookScript(f,"OnHide","NOOP")
        HideUIPanel(f)
    end
    
    UIPanelWindows[n] = info
    
    if(visible) then
        ShowUIPanel(f)
        self:UnhookScript(f,"OnShow")
        self:UnhookScript(f,"OnHide")
    end
end

function FilterTradeSkill:OnEvent(event,arg1)
    if(event == "ADDON_LOADED" and arg1 == "FilterTradeSkill") then
        self.db = FilterTradeSkillDB
        self:SetDefaults()
        self:ApplySettings()
        FilterTradeSkillCheckButton.db = self.db
        FilterTradeSkillSlider.db = self.db
        self:CreateDDConfig()
        self:UpdateFilter()
        --TradeSkillFrame_Update()
    elseif(event == "TRADE_SKILL_SHOW") then
        self:UpdateFilter()
    end
end


function FilterTradeSkill:Resize(newsize)
    
    newsize = newsize or self.db.ShownSkills
    TRADE_SKILLS_DISPLAYED = self.db.ShownSkills
    TradeSkillFrame:SetHeight(424+newsize*16)

    TradeSkillListScrollFrame:SetHeight(2+newsize*16);
    TradeSkillListScrollFrameMiddleSection:SetHeight(16*newsize-60);
    
    TradeSkillHorizontalBarLeft:SetPoint("TOPLEFT","TradeSkillFrame","TOPLEFT",15,-251-(newsize-8)*16);
    TradeSkillDetailScrollFrame:SetPoint("TOPLEFT","TradeSkillFrame","TOPLEFT",20,-262-(newsize-8)*16);

    TradeSkillCreateButton:SetPoint("CENTER","TradeSkillFrame","TOPLEFT",224,-462-(newsize-8)*16);
    TradeSkillCancelButton:SetPoint("CENTER","TradeSkillFrame","TOPLEFT",305,-462-(newsize-8)*16);

    if(newsize < 14) then
        self.topleft:SetTexture("Interface\\Addons\\FilterTradeSkill\\artwork\\top-left-short");
        self.topright:SetTexture("Interface\\Addons\\FilterTradeSkill\\artwork\\top-right-short")
    else
        self.topleft:SetTexture("Interface\\Addons\\FilterTradeSkill\\artwork\\top-left");
        self.topright:SetTexture("Interface\\Addons\\FilterTradeSkill\\artwork\\top-right")
    end
    
    for i = newsize+1,20 do
        getglobal("TradeSkillSkill"..i):Hide()
    end

    TradeSkillFrame_Update()
    
    -- Adjust AutoCraftBackground's height when I change mine
    if(AutoCraftBackground) then
        AutoCraftBackground:SetHeight(TradeSkillFrame:GetHeight() + 48)
    end
end

function FilterTradeSkill:CreateDDConfig()
    local ddconfig = {
        {
            text = "Filter Fields",
            notClickable = true,
            isTitle = true,
        },
        {text = "Name", op = "name"},
        {text = "Description", op = "desc"},
        {text = "Components", op = "mats"},
        {
            text = "Skill Difficulty",
            notClickable = true,
            isTitle = true,
        },
        {text = "Show Optimal", op = "optimal"},
        {text = "Show Medium", op = "medium"},
        {text = "Show Easy", op = "easy"},
        {text = "Show Trivial", op = "trivial"},
    }
    local f = function() self:DropDownOnClick() end
    for i,v in pairs(ddconfig) do
        if( not v.isTitle) then
            v.keepShownOnClick = true
            v.func = f
        end
    end

    self.ddconfig = ddconfig
    UIDropDownMenu_Initialize(FilterTradeSkillDropDown,function() self:InitDropDown() end)
    FilterTradeSkillDropDownText:SetText("Filter Options");
end

function FilterTradeSkill:InitDropDown()
    for i,v in pairs(self.ddconfig) do
        if(not v.isTitle) then v.checked = self.db[v.op] end
        UIDropDownMenu_AddButton(v)
    end
end

function FilterTradeSkill:DropDownOnClick()
    local id = this:GetID()
    local op = self.ddconfig[id].op
    self.db[op] = not self.db[op]
    self:UpdateFilter()
    TradeSkillFrame_Update()
end

function FilterTradeSkill:ToggleOptionsPane()
    local f = FilterTradeSkillConfigFrame
    if(f:IsVisible()) then
        f:Hide()
    else
        f:Show()
    end
end

function FilterTradeSkill:ToggleLock()
    self.db.locked = not self.db.locked
    self:UpdateLockButton()
end

function FilterTradeSkill:UpdateLockButton()
    
    local button = self.lockbutton
    button:Enable()
    if(not self.db.movable) then
        self.db.locked = true
        button:Disable()
    end

    if(self.db.locked) then
        button.normal:SetTexture("Interface\\Addons\\FilterTradeSkill\\artwork\\LockButton-Locked-Up")
        button.pushed:SetTexture("Interface\\Addons\\FilterTradeSkill\\artwork\\LockButton-Locked-Down")
    else
        button.normal:SetTexture("Interface\\Addons\\FilterTradeSkill\\artwork\\LockButton-Unlocked-Up")
        button.pushed:SetTexture("Interface\\Addons\\FilterTradeSkill\\artwork\\LockButton-Unlocked-Down")
    end
end


function FilterTradeSkill:SetFilter(newfilter)
    self.filter = string.lower(newfilter)
    self:UpdateFilter()
    TradeSkillFrame_Update()
end

function FilterTradeSkill:UpdateFilter()
    if(not TradeSkillFrame:IsVisible()) then return end
    --self:Print("Update Filter")
    self.updatingFilter = true
    local headers = {}
    for i=1,GetNumTradeSkills() do
        local name,skilltype,_,expanded = GetTradeSkillInfo(i)
        if(skilltype == "header") then
            headers[name] = expanded and true or false
        end
    end
    
    ExpandTradeSkillSubClass(0)
    
    local showState = self.showState
    local numAF,show,curheader = 0,false
    
    for i=1,GetNumTradeSkills() do
        show = false
        local name,skilltype = GetTradeSkillInfo(i)
        if(skilltype == "header") then
            curheader = name
            showState[name] = false
        else
            if(self.db[skilltype]) then
                if(self.filter == "") then
                    show = true
                elseif(self.db.name and string.find(string.lower(name),self.filter,nil,self.db.noregex)) then
                    show =  true
                elseif(self.db.mats and self:SearchMats(i)) then
                    show = true			
                elseif(self.db.desc and self:SearchDescription(i)) then
                    show = true
                end
            end
            if(show) then
                showState[name] = true
                if(curheader) then showState[curheader] = true end
                if(headers[curheader]) then numAF = numAF + 1 end
            else
                showState[name] = false
            end			
        end
    end
    
    for i=GetNumTradeSkills(),1,-1 do
        local n,t = GetTradeSkillInfo(i)
        if(t == "header" and (not headers[n])) then
            CollapseTradeSkillSubClass(i)
        end
    end
    
    for h in pairs(headers) do
        if(showState[h]) then
            numAF = numAF + 1
        end
    end
    
    if(numAF == 0) then
        --TODO play sound/editbox background
        TradeSkillCollapseAllButton:Disable()
    else
        TradeSkillCollapseAllButton:Enable()
    end
    self.numAF = numAF
    self.showState = showState
    self.updatingFilter = nil
    self.currentSkill = GetTradeSkillLine()
end

function FilterTradeSkill:Update()
    if(not self.db) then return end
    if(self.updatingFilter) then return end
    
    if(self.currentSkill ~= (GetTradeSkillLine())) then self:UpdateFilter() end
    
    -- This is almost a direct copy/paste from the Blizzard function TradeSkillFrame_Update 
    local numTradeSkills = GetNumTradeSkills()
    
    --ChatFrame3:AddMessage(debugstack())
    --self:Print("Update")
    
    FauxScrollFrame_Update(TradeSkillListScrollFrame, self.numAF or 0, self.db.ShownSkills, TRADE_SKILL_HEIGHT, nil, nil, nil, TradeSkillHighlightFrame, 293, 316 )
    local skillOffset = FauxScrollFrame_GetOffset(TradeSkillListScrollFrame)
    
    if ( numTradeSkills == 0 ) then
        TradeSkillFrameTitleText:SetText(format(TEXT(TRADE_SKILL_TITLE), GetTradeSkillLine()))
        TradeSkillSkillName:Hide()
        TradeSkillSkillIcon:Hide()
        TradeSkillRequirementLabel:Hide()
        
        for i=1, MAX_TRADE_SKILL_REAGENTS, 1 do
            getglobal("TradeSkillReagent"..i):Hide()
        end
    else
        TradeSkillSkillName:Show()
        TradeSkillSkillIcon:Show()
        
    end
    
    TradeSkillHighlightFrame:Hide()
    
    local skillIndex = 0
    local count = 0
    
    for i=1, self.db.ShownSkills do

        local showthis = false
        local skillName, skillType, numAvailable, isExpanded
        
        while((showthis == false) and (skillIndex <= numTradeSkills)) do
        
            skillIndex = skillIndex + 1
            skillName, skillType, numAvailable, isExpanded = GetTradeSkillInfo(skillIndex)
            if (self.showState or SHOWSTATE_ALLTRUE)[skillName] then
                count = count + 1
                if count > skillOffset then
                    showthis = 1
                else
                    showthis = false
                end
            end
        end
    
        local skillButton = getglobal("TradeSkillSkill"..i)
        if ( skillIndex <= numTradeSkills ) then	
            
            if ( TradeSkillListScrollFrame:IsVisible() ) then
                skillButton:SetWidth(293)
            else
                skillButton:SetWidth(323)
            end
            local color = TradeSkillTypeColor[skillType]
            if ( color ) then
                skillButton:SetTextColor(color.r, color.g, color.b)
            end
            
            skillButton:SetID(skillIndex)
            skillButton:Show()
            
            if ( skillType == "header" ) then
                skillButton:SetText(skillName)
                if ( isExpanded ) then
                    skillButton:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up")
                else
                    skillButton:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up")
                end
                getglobal("TradeSkillSkill"..i.."Highlight"):SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight")
                getglobal("TradeSkillSkill"..i):UnlockHighlight()
            else
                if ( not skillName ) then
                    return
                end
                skillButton:SetNormalTexture("")
                getglobal("TradeSkillSkill"..i.."Highlight"):SetTexture("")
                if ( numAvailable == 0 ) then
                    skillButton:SetText(" "..skillName)
                else
                    skillButton:SetText(" "..skillName.." ["..numAvailable.."]")
                end
                
                if ( GetTradeSkillSelectionIndex() == skillIndex ) then
                    TradeSkillHighlightFrame:SetPoint("TOPLEFT", "TradeSkillSkill"..i, "TOPLEFT", 0, 0)
                    TradeSkillHighlightFrame:Show()
                    TradeSkillFrame.numAvailable = numAvailable
                    getglobal("TradeSkillSkill"..i):LockHighlight()
                else
                    getglobal("TradeSkillSkill"..i):UnlockHighlight()
                end
            end
            
        else
            skillButton:Hide()
        end
    end
    
    local numHeaders = 0
    local notExpanded = 0
    for i=1, numTradeSkills, 1 do
        local index = i + skillOffset
        local skillName, skillType, numAvailable, isExpanded = GetTradeSkillInfo(index)
        if ( skillName and skillType == "header" ) then
            numHeaders = numHeaders + 1
            if ( not isExpanded ) then
                notExpanded = notExpanded + 1
            end
        end
    end
    
    if ( notExpanded ~= numHeaders ) then
        TradeSkillCollapseAllButton.collapsed = nil
        TradeSkillCollapseAllButton:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up")
    else
        TradeSkillCollapseAllButton.collapsed = 1
        TradeSkillCollapseAllButton:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up")
    end
end



function FilterTradeSkill:SearchMats(i)
    local show = false
    local numReagents = GetTradeSkillNumReagents(i)
    for j=1, numReagents do
        local reagentName,_,_,_ = GetTradeSkillReagentInfo(i,j)
        if(string.find(string.lower(reagentName or ""),self.filter,nil,self.db.noregex)) then
            show = true
            break
        end
    end

    return show
end

function FilterTradeSkill:SearchDescription(i)
    local tooltip = self.tooltip
    tooltip:ClearLines()
    tooltip:SetOwner(UIParent,"ANCHOR_NONE")
    tooltip:SetTradeSkillItem(i)
    local show = false
    local leftname = tooltip:GetName().."TextLeft"
    for i=2, tooltip:NumLines() do

        local desc = getglobal(leftname..i):GetText()
        if(string.find(desc,"^Use:")) then
            desc = string.gsub(desc,"^Use: ","")
        end

        if(string.find(string.lower(desc),self.filter)) then
            show = true
            break
        end
    end

    return show
end
function FilterTradeSkill:OnMouseDown()
    if(self.db.movable and (not self.db.locked)) then
        this:StartMoving()
        self.moving = true
    end
    self:CallScript(this,"OnMouseDown")
end

function FilterTradeSkill:OnMouseUp()
    if(self.moving) then
        self:StopFrame(this)
        self.moving = nil
    end
    self:CallScript(this,"OnMouseDown")
end

function FilterTradeSkill:OnHide()
    if(self.moving) then
        self:StopFrame()
    end
    --self:Print("OnHide")
    HideUIPanel(FilterTradeSkillAnchorFrame)
    --self:CallScript(this,"OnHide")
end
function FilterTradeSkill:OnShow()
    self:UpdateFilter()
	--self:Print("OnShow")
    TradeSkillFrame_Update()
    ShowUIPanel(FilterTradeSkillAnchorFrame)
    if(self.db.autoFocus) then
        self.editbox:SetFocus()
    end
    --self:CallScript(this,"OnShow")
end
function FilterTradeSkill:StopFrame(f)
    f:StopMovingOrSizing()
    local x,y,s = f:GetLeft(),f:GetTop(),f:GetEffectiveScale()
    x,y = x*s,y*s
    self.db.pos = {x,y}
end
function FilterTradeSkill:ResetFramePosition()
    local pos = self.db.pos
    if(not pos) then return end
    TradeSkillFrame:ClearAllPoints()
    TradeSkillFrame:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT",pos[1],pos[2])
end
function FilterTradeSkill:ClearFramePosition()
    self.db.pos = nil
end
function FilterTradeSkill:ExpandTradeSkillSubClass(id)
    self.Hooks.ExpandTradeSkillSubClass.orig(id)
    if(self.updatingFilter) then return end
    
    self:UpdateFilter()
    TradeSkillFrame_Update()
end

function FilterTradeSkill:CollapseTradeSkillSubClass(id)
    self.Hooks.CollapseTradeSkillSubClass.orig(id)
    if(self.updatingFilter) then return end
    
    self:UpdateFilter()
    TradeSkillFrame_Update()
end

function FilterTradeSkill:Print(msg)
    ChatFrame1:AddMessage("|Cff40ff40FilterTadeSkill: "..tostring(msg))
end
FilterTradeSkill:OnLoad()